/*
*   Visual Basic Callback Server Source
*
*   (c) Copyright Microsoft Corp. 1995 All Rights Reserved
*/ 

/*
This file is used to generate the callback server.  This is the only
file you should have to modify (besides changing GUIDs in the ODL) to
create your own version of a callback server.  All you have to do is define the
fields shown below and include CBackMap.H, which does all the work.

CBField_Name  	     Name of the callback, eg WNDPROC Required
CBField_ReturnType   ReturnType for callback. void if Type = VOID.  Default void
CBField_ParamList    Full parameter list without (), must include parameter names. Required
CBField_CallList     Full call list without () unless Type = VOID. Required.
CBField_Help16       Quoted helpstring for 16 bit.  Gives first class member prototype. default ""
CBField_Help32       Modified Help16													default ""
CBCHECKBREAK         Block any calls and use DebugProc if VB is in break mode.
CBPERSISTENT         Callback is persistent if defined (32 instances)
CBVOID               Callback return type is void if defined*/

#ifndef WIN32
#define LPTSTR LPSTR
#define LPCTSTR LPCSTR
#endif

#define CBField_Name WNDPROC
#define CBField_ReturnType LRESULT
#define CBField_ParamList HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam
#define CBField_CallList hWnd, msg, wParam, lParam
#define CBPERSISTENT
#define CBCHECKBREAK
#define CBField_Help16 "(ByVal Integer, ByVal Integer, ByVal Integer, ByVal Long) As Long"
#define CBField_Help32 "(ByVal Long, ByVal Long, ByVal Long, Byval Long) As Long"
#include "CBackMap.H"																				   

#define CBField_Name DLGPROC
#define CBField_ReturnType LRESULT
#define CBField_ParamList HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam
#define CBField_CallList hWnd, msg, wParam, lParam
#define CBPERSISTENT
#define CBCHECKBREAK
#define CBField_Help16 "(ByVal Integer, ByVal Integer, ByVal Integer, ByVal Long) As Long"
#define CBField_Help32 "(ByVal Long, ByVal Long, ByVal Long, ByVal Long) As Long"
#include "CBackMap.H"																				   

#define CBField_Name HOOKPROC
#define CBField_ReturnType LRESULT
#define CBField_ParamList int code, WPARAM wParam, LPARAM lParam
#define CBField_CallList code, wParam, lParam
#define CBPERSISTENT
#define CBCHECKBREAK
#define CBField_Help16 "(ByVal Integer, ByVal Integer, ByVal Long) As Long"
#define CBField_Help32 "(ByVal Long, ByVal Long, ByVal Long) As Long"
#include "CBackMap.H"                                         

#define CBField_Name WNDENUMPROC
#define CBField_ReturnType BOOL
#define CBField_ParamList HWND hWnd, LPARAM lParam
#define CBField_CallList hWnd, lParam
#define CBField_Help16 "(ByVal Integer, Any) As Integer"
#define CBField_Help32 "(ByVal Long, Any) As Long"
#include "CBackMap.H"                                                                                

#define CBField_Name FONTENUMPROC
#define CBField_ReturnType int
#define CBField_ParamList ENUMLOGFONT FAR* elf, NEWTEXTMETRIC FAR* ntm, int sysint, LPARAM lParam
#define CBField_CallList elf, ntm, sysint, lParam
#define CBField_Help16 "(ByVal Long (Address), ByVal Long (Address), ByVal Integer, Any) As Integer"
#define CBField_Help32 "(ByVal Long (Address), ByVal Long (Address), ByVal Long, Any) As Long"
#include "CBackMap.H"																				   

#define CBField_Name SENDASYNCPROC
#define CBField_ParamList HWND hWnd, UINT uint, DWORD dword, LRESULT lresult
#define CBField_CallList hWnd, uint, dword, lresult
#define CBField_Help16 "(ByVal Integer, ByVal Integer, ByVal Long, ByVal Long)"
#define CBField_Help32 "(ByVal Long, ByVal Long, ByVal Long, ByVal Long)"
#define CBVOID
#include "CBackMap.H"																				   

#define CBField_Name TIMERPROC
#define CBField_ParamList HWND hWnd, UINT msg, UINT idTimer, DWORD dwTime
#define CBField_CallList hWnd, msg, idTimer, dwTime
#define CBPERSISTENT
#define CBCHECKBREAK
#define CBVOID
#define CBField_Help16 "(ByVal Integer, ByVal Integer, ByVal Integer, ByVal Long)"
#define CBField_Help32 "(ByVal Long, ByVal Long, ByVal Long, ByVal Long)"
#include "CBackMap.H"																				   

#define CBField_Name GOBJENUMPROC
#define CBField_ParamList LPVOID lpLogObject, LPARAM lpData
#define CBField_CallList lpLogObject, lpData
#define CBVOID
#define CBField_Help16 "(ByVal Long (Address), Any)"
#define CBField_Help32 "(ByVal Long (Address), Any)"
#include "CBackMap.H"																				   

#define CBField_Name PROPENUMPROC
#define CBField_ReturnType BOOL
#define CBField_ParamList HWND hwnd, LPCTSTR lpszString, HANDLE hData
#define CBField_CallList hwnd, lpszString, hData
#define CBField_Help16 "(ByVal Integer, ByVal Long (LPSTR), ByVal Integer) As Integer"
#define CBField_Help32 "(ByVal Long, ByVal Long (LPSTR), ByVal Long) As Long"
#include "CBackMap.H"																				   

#define CBField_Name MFENUMPROC
#define CBField_ReturnType int
#define CBField_ParamList HDC hDC, HANDLETABLE FAR *lpHTable, METARECORD FAR* lpMFR, int nObj, LPARAM lpClientData
#define CBField_CallList hDC, lpHTable, lpMFR, nObj, lpClientData
#define CBField_Help16 "(ByVal Long, ByVal Long (Address), ByVal Long (Address), ByVal Integer, Any) As Integer"
#define CBField_Help32 "(ByVal Long, ByVal Long (Address), ByVal Long (Address), ByVal Long, Any) As Long"
#include "CBackMap.H"																				   

#define CBField_Name GRAYSTRINGPROC
#define CBField_ReturnType BOOL
#define CBField_ParamList HDC hDC, LPARAM lpData, int cchData
#define CBField_CallList hDC, lpData, cchData
#define CBCHECKBREAK
#define CBField_Help16 "(ByVal Long, ByVal Long (LPSTR), ByVal Integer) As Integer"
#define CBField_Help32 "(ByVal Long, ByVal Long (LPSTR), ByVal Long) As Long"
#include "CBackMap.H"																				   

#define CBField_Name EDITWORDBREAKPROC
#define CBField_ReturnType int
#define CBField_ParamList LPTSTR lpch, int ichCurrent, int cch, int code
#define CBField_CallList lpch, ichCurrent, cch, code
#define CBCHECKBREAK
#define CBField_Help16 "(ByVal Long (LPSTR), ByVal Integer, ByVal Integer, ByVal Integer) As Integer"
#define CBField_Help32 "(ByVal Long (LPSTR), ByVal Long, ByVal Long, ByVal Long) As Long"
#include "CBackMap.H"																				   

#define CBField_Name LINEDDAPROC
#define CBField_ParamList int X, int Y, LPARAM lpData 
#define CBField_CallList X, Y, lpData
#define CBVOID
#define CBCHECKBREAK
#define CBField_Help16 "(ByVal Integer, ByVal Integer, Any)"
#define CBField_Help32 "(ByVal Long, ByVal Long, Any)"
#include "CBackMap.H"																				   

#define CBField_Name AVICapStatus
#define CBField_ReturnType LRESULT
#define CBField_ParamList HWND hWnd, int sysint, LPWSTR lpwstr
#define CBField_CallList hWnd, sysint, lpwstr
#define CBField_Help16 "(ByVal Integer, ByVal Integer, String) As Integer"
#define CBField_Help32 "(ByVal Long, ByVal Long, String) As Long"
#include "CBackMap.H"																				   

#define CBField_Name AVICapFrame
#define CBField_ReturnType LRESULT
#define CBField_ParamList HWND hWnd, long lng
#define CBField_CallList hWnd, lng
#define CBField_Help16 "(ByVal Integer, ByVal Integer) As Integer"
#define CBField_Help32 "(ByVal Long, ByVal Long) As Long"
#include "CBackMap.H"		 																		   

#define CBField_Name AVICapError
#define CBField_ReturnType LRESULT
#define CBField_ParamList HWND hWnd, int sysint, LPWSTR lpwstr
#define CBField_CallList hWnd, sysint, lpwstr
#define CBField_Help16 "(ByVal Integer, ByVal Integer, String) As Integer"
#define CBField_Help32 "(ByVal Long, ByVal Long, String) As Long"
#include "CBackMap.H"

#ifdef WIN32
#define CBField_Name PROPENUMPROCEX
#define CBField_ReturnType BOOL
#define CBField_ParamList HWND hwnd, LPCTSTR lpszString, HANDLE hData, DWORD dwData
#define CBField_CallList hwnd, lpszString, hData, dwData
#define CBField_Help16 "(ByVal Integer, ByVal Long (LPSTR), ByVal Integer, Any) As Integer"
#define CBField_Help32 "(ByVal Long, ByVal Long (LPSTR), ByVal Long, Any) As Long"
#include "CBackMap.H"																				   

#define CBField_Name ENUMRESLANGPROC
#define CBField_ReturnType BOOL
#define CBField_ParamList HANDLE hModule, LPCTSTR lpszType, LPCTSTR lpszName, WORD wIDLanguage, LONG lParam
#define CBField_CallList hModule, lpszType, lpszName, wIDLanguage, lParam
#define CBField_Help32 "(ByVal Long, ByVal Long (LPSTR), ByVal Long (LPSTR), ByVal Long, Any) As Long"
#include "CBackMap.H"																				   

#define CBField_Name ENUMRESNAMEPROC
#define CBField_ReturnType BOOL
#define CBField_ParamList HANDLE hModule, LPCTSTR lpszType, LPCTSTR lpszName, LONG lParam
#define CBField_CallList hModule, lpszType, lpszName, lParam
#define CBField_Help32 "(ByVal Long, ByVal Long (LPSTR), ByVal Long (LPSTR), Any) As Long"
#include "CBackMap.H"																				   

#define CBField_Name ENUMRESTYPEPROC
#define CBField_ReturnType BOOL
#define CBField_ParamList HANDLE hModule, LPCTSTR lpszType, LONG lParam
#define CBField_CallList hModule, lpszType, lParam
#define CBField_Help32 "(ByVal Long, ByVal Long (LPSTR), Any) As Long"
#include "CBackMap.H"																				   
#endif


