Attribute VB_Name = "MExeType"
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

'$ Uses UTILITY.BAS BYTES.BAS

' Valid Exe types (for ExeType function)
Public Const ordMSDOS = 1
Public Const ordWin16 = 2
Public Const ordOS2_1 = 3
Public Const ordWin32 = 4
Public Const ordWin32Console = 5
Public Const ordDOSUnknown = 7
Public Const sMSDOS = "MS-DOS"
Public Const sWin16 = "Windows 16-bit"
Public Const sOS2_1 = "OS/2 1.x"
Public Const sWin32 = "Windows 32-bit"
Public Const sWin32Console = "Windows 32-bit Console"
Public Const sDOSUnknown = "Unknown MS-DOS Compatible"

' Unknown - could still be .BAT, .CMD, .COM, or .P
Public Const ordNotExe = 0
Public Const sNotExe = "Not EXE File"

' Errors
Public Const errNoFile = -1
Public Const errOS2_2 = -2
Public Const errWinOS2DLL = -3
Public Const errNEUnknown = -4
Public Const errNTNonIntel = -5
Public Const errWin32DLL = -6
Public Const errAccessFail = -7

Public Const sNoFile = "No Such File"
Public Const sOS2_2 = "OS/2 2.x or Greater"
Public Const sWinOS2DLL = "Windows 3.x or OS/2 DLL"
Public Const sNEUnknown = "Unknown Format"
Public Const sNTNonIntel = "Non-Intel Windows"
Public Const sWin32DLL = "Windows 32-bit DLL"


' Check to see if specified file is executable, and if so, what kind
Function ExeType(sSpec As String) As Integer
    On Error GoTo ExeTypeFail
    Dim hFile As Integer
    hFile = FreeFile
    If ExistFile(sSpec) Then
        Open sSpec For Binary Access Read Shared As hFile
    Else
        ExeType = errNoFile
        Exit Function
    End If

    Dim abHeader() As Byte
    ReDim abHeader(128)
    Get hFile, 1, abHeader

    ' MS-DOS headers start with magic header "MZ"
    Dim sMagic As String, bData As Byte, wData As Integer
    sMagic = LeftBytes(abHeader, 2)
    If sMagic <> "MZ" Then
        ' Could still be a .BAT, .CMD, .PIF, or .COM file
        ' but that's not our problem here
        ExeType = ordNotExe
        Close hFile
        Exit Function
    End If

    ' If word at offset &H18 does not point beyond DOS header
    ' (length &H40), file is MS-DOS EXE
    If BytesToWord(abHeader, &H18) < &H40 Then
        ExeType = ordMSDOS
        Close hFile
        Exit Function
    End If

    ' Get offset of new EXE header
    wData = BytesToWord(abHeader, &H3C)
    Get hFile, wData + 1, abHeader
    Close hFile

    ' New .EXE headers start with magic header "NE"
    sMagic = LeftBytes(abHeader, 2)
    ' Check for Windows/OS2 format
    If sMagic = "NE" Then

        ' Get the executable file flags to check for DLL
        If abHeader(&HD) And &H80 Then
            ' This is a DLL (executable but not by us)
            ExeType = errWinOS2DLL
        Else
            ' Get the operating system flags (byte, not word)
            bData = abHeader(&H36)
            If bData And &H2 Then
                ExeType = ordWin16 ' Windows
            ElseIf bData And &H1 Then
                ExeType = ordOS2_1 ' OS/2 1.x
            Else
                ' Unknown NE system, probably bound, but call it MS-DOS
                ExeType = ordMSDOS
            End If
        End If

    ' Check for OS/2 2.x format (can't execute from Windows or NT)
    ElseIf sMagic = "LE" Then
        ExeType = errOS2_2 ' OS/2 LE
    ' Check for NT format
    ElseIf sMagic = "PE" And BytesToWord(abHeader, &H2) = 0 Then
        ' Get processor flags
        bData = abHeader(&H4)
        Select Case bData
        Case &H4C, &H4D, &H4E, &H4F ' NT for intel 386, 486, 586, 686
            ExeType = ordWin32 ' NT Windows
        Case Else
            ExeType = errNTNonIntel ' Some sort of RISC or other
            Exit Function
        End Select

        ' Get the Exe type flags
        If abHeader(&H17) And &H20 Then
            ExeType = errWin32DLL ' Executable, but not by us
            Exit Function
        End If

        ' Get the subsystem flags to identify NT character
        If abHeader(&H5C) = 3 Then ExeType = ordWin32Console
        ' Could also identify Posix here

    Else
        ' MS-DOS file with a header, but notNE file
        ' (Some 16-bit DOS-extended executables fall through here, or
        ' could be non-EXE file with "MZ" as first two bytes)
        ExeType = ordDOSUnknown ' Probably DOS extended
    End If
    Exit Function
    
ExeTypeFail:
    ExeType = errAccessFail
End Function

Function ExeTypeStr(sFile As String) As String
    Select Case ExeType(sFile)
    ' Valid Exe types (for ExeType function)
    Case ordMSDOS
        ExeTypeStr = sMSDOS
    Case ordWin16
        ExeTypeStr = sWin16
    Case ordOS2_1
        ExeTypeStr = sOS2_1
    Case ordWin32
        ExeTypeStr = sWin32
    Case ordWin32Console
        ExeTypeStr = sWin32Console
    Case ordDOSUnknown
        ExeTypeStr = sDOSUnknown
    Case ordNotExe
        ExeTypeStr = sNotExe
    Case errNoFile
        ExeTypeStr = sNoFile
    Case errOS2_2
        ExeTypeStr = sOS2_2
    Case errWinOS2DLL
        ExeTypeStr = sWinOS2DLL
    Case errNEUnknown
        ExeTypeStr = sNEUnknown
    Case errNTNonIntel
        ExeTypeStr = sNTNonIntel
    Case errWin32DLL
        ExeTypeStr = sWin32DLL
    End Select
End Function


