Attribute VB_Name = "MFindFile"
Option Explicit

Function FindFilesDir(sTarget As String, Optional vStart As Variant) As Variant

    ' Use statics for better performance and to use less
    ' disk space in recursive routine
    Static sName As String, sSpec As String
    Static nFound As New Collection, fNotFirstTime As Boolean
    Dim nDirNames As New Collection, sStart As String
    If IsMissing(vStart) Then sStart = CurDir$ Else sStart = vStart
    sStart = NormalizePath(sStart)
    If fNotFirstTime Then
        Set nFound = Nothing
        fNotFirstTime = True
    End If
    sStart = NormalizePath(sStart)
    
    ' Ignore errors so that VB invalid file name won't kill search
    ' (Basic fails on weird but legal Win32 names such as ??????)
    On Error Resume Next
    
    ' Get first name
    sName = Dir$(sStart, vbDirectory)
    
    ' Iterate through
    Do While sName <> sEmpty
        If sName <> "." Then
            If sName <> ".." Then
                sSpec = sStart & sName
                If GetAttr(sSpec) And vbDirectory Then
                    ' Cache directory names
                    nDirNames.Add sName
                ElseIf StrComp(sName, sTarget, 1) = 0 Then ' Text comparison
                    ' Store file names in result
                    nFound.Add sSpec
                End If
            End If
        End If
        ' Get next name
        sName = Dir$()
    Loop

    ' Recursively iterate through each cached directory
    Static v As Variant
    For Each v In nDirNames
        FindFilesDir sTarget, sStart & v
    Next

    ' Return the count of matching files
    Set FindFilesDir = nFound
    Exit Function

End Function

#If Win32 Then
Function FindFiles(sTarget As String, Optional vStart As Variant) As Variant

    ' Use statics for better performance and to use less
    ' disk space in recursive routine
    Static sName As String, sSpec As String
    Static nFound As New Collection
    Static fd As TFindData, f As Boolean, fNotFirstTime
    Dim nDirNames As New Collection, sStart As String, hFiles As Long
    If IsMissing(vStart) Then sStart = CurDir$ Else sStart = vStart
    If fNotFirstTime Then
        Set nFound = Nothing
        fNotFirstTime = True
    End If
    sStart = NormalizePath(sStart)
    
    hFiles = FindFirstFile(sStart & "*.*", fd)
    f = (hFiles <> INVALID_HANDLE_VALUE)
    Do While f
        If Left$(fd.cFileName, 1) <> "." Then
            sName = StrZToStr(fd.cFileName)
            sSpec = sStart & sName
            If fd.dwFileAttributes And vbDirectory Then
                ' Call recursively check directory
                DoEvents
                FindFiles sTarget, sSpec
            ElseIf StrComp(sName, sTarget, 1) = 0 Then ' Text comparison
                ' Store file names in result
                nFound.Add sSpec
            End If
        End If
        
        ' Keep looping until a file matches the attributes
        f = FindNextFile(hFiles, fd)
    Loop
    f = FindClose(hFiles)
    
    ' Return the count of matching files
    Set FindFiles = nFound

End Function
#Else
Function FindFiles(sTarget As String, Optional vStart As Variant) As Variant
    FindFiles = FindFilesDir(sTarget, vStart)
End Function
#End If

#If 0 Then
''@B Files
Function DirFiles(sSpec As String, hFiles As Long, _
                  Optional vInAttr As Variant) As String
    Dim fd As TFindData, sName As String, f As Boolean
    Dim sPath As String, afAttr As Long
    
    ' Stop finding and close handle early
    If IsMissing(vInAttr) Then afAttr = vbNormal Else afAttr = vInAttr
    If afAttr = vbStopFind Then
        f = FindClose(hFiles)
        hFiles = 0: Exit Function
    End If
    f = True
    Do
        ' Get first or next file
        If hFiles = 0 Then
            hFiles = FindFirstFile(sSpec, fd)
        Else
            f = FindNextFile(hFiles, fd)
        End If
        If (f = False Or hFiles = INVALID_HANDLE_VALUE) Then
            If Err.LastDllError = ERROR_NO_MORE_FILES Then
                f = FindClose(hFiles)
            End If
            hFiles = 0: Exit Function
        End If
        ' Keep looping until a file matches the attributes
    Loop While (afAttr <> vbNormal) And _
               ((afAttr And fd.dwFileAttributes) = 0)
    ' Get the file data and return through reference
    sPath = GetFileDir(sSpec)
    sName = StrZToStr(fd.cFileName)
    fi.CreateFile sPath & sName, fd.dwFileAttributes, fd.nFileSizeLow, _
                  SysToVBTime(fd.ftLastWriteTime), _
                  SysToVBTime(fd.ftLastAccessTime), _
                  SysToVBTime(fd.ftCreationTime)
    Files = sName
End Function
''@E Files

#End If

