VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CMenuList"
Attribute VB_Creatable = True
Attribute VB_Exposed = False
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

''@B Internal
Private nItems As New Collection
Private hMenu As Long, idMax As Long
' Warning! These members should be static--that is, one variable
' available to all objects of the given type. The entire menu tree
' has only one window handle. But Basic doesn't have static
' variables, so you must fake them.
Private hWnd As Long, fSys As Boolean
' Warning! Violation of encapsulation standards! This property
' should not be public, but its partner class won't work otherwise.
' Don't use this property directly.
Public Parent As CMenuList
''@E Internal

Sub Class_Initialize()
    ' Make sure Parent property is initialized
    Set Parent = Nothing
End Sub

Sub Class_Terminate()
    DestroyMenus
End Sub

' Call this by passing window handle (it can take menu handle, but
' should never get one through caller)
''@B Create1
Function Create(hA As Long, Optional vSys As Variant) As Boolean
    If IsMissing(vSys) Then fSys = False Else fSys = CBool(vSys)
    If IsWindow(hA) Then
        ' Create system or normal menu from hWnd
        If fSys Then
            hMenu = GetSystemMenu(hA, False)
        Else
            hMenu = GetMenu(hA)
        End If
        hWnd = hA
    Else
        ' Don't accept menu handle from top node
        If IsMenu(hA) And Parent Is Nothing Then Exit Function
        hMenu = hA: hWnd = WinHandle
    End If
''@E Create1

''@B Create2
    ' Create each item in list and add to collection
    Dim ritem As CMenuItem, i As Long, f As Boolean
    DestroyMenus
    For i = 0 To Count - 1
        Set ritem = New CMenuItem
        ' Create will also create new submenus
        f = ritem.Create(i, hMenu, Me)
        BugAssert f     ' Should never fail
        nItems.Add ritem
        ' Needed by InsertNew method
        If ritem.ID > idMax Then idMax = ritem.ID
    Next
    Create = True
End Function
''@E Create2

' Redraw after each change
Sub ReDraw()
    DrawMenuBar hWnd
End Sub

' Insert new item, moving everything down
Function InsertNew(sItem As String, Optional iPos As Variant, _
                   Optional afFlags As Variant) As Boolean
    If IsMissing(iPos) Then iPos = Count   ' Append to end
    If IsMissing(afFlags) Then afFlags = MF_STRING Or MF_DISABLED
    InsertNew = InsertMenu(hMenu, iPos, MF_BYPOSITION Or afFlags, _
                           idMax + 10, sItem)
    Dim f As Boolean, ritem As CMenuItem
    f = ritem.Create(CInt(iPos), hMenu, Me)
    BugAssert f     ' Should never fail
    nItems.Add ritem
    If ritem.ID > idMax Then idMax = ritem.ID
End Function

' Move up through recursive levels and recreate window from top
Function Refresh()
    If Parent Is Nothing Then
        Refresh = Create(hWnd, fSys)
    Else
        Refresh = Parent.Refresh
    End If
End Function

' Number of items in menu
Property Get Count() As Integer
    Count = GetMenuItemCount(hMenu)
    If Count = -1 Then Count = 0
End Property

' Move up through recursive levels to find top level window handle
' (static member would be much easier)
''@B WinHandle
Property Get WinHandle() As Long
    If Parent Is Nothing Then
        WinHandle = hWnd
    Else
        WinHandle = Parent.WinHandle
    End If
End Property
''@E WinHandle

Property Get SysMenu() As Boolean
    If Parent Is Nothing Then
        SysMenu = fSys
    Else
        SysMenu = Parent.SysMenu
    End If
End Property

' Clear everything from here on down
Public Sub DestroyMenus()
    Dim ritem As CMenuItem
    For Each ritem In nItems
        If ritem.Popup Then ritem.Child.DestroyMenus
        Set ritem = Nothing
        nItems.Remove 1
    Next
End Sub

' Find menu item by its string name
Function Find(sName As String, ritem As CMenuItem) As Boolean
    Find = True
    Dim i As Integer
    ' Step through each item, searching for match
    For Each ritem In nItems
        ' Test against name (stripped version of text)
        If ritem.Name Like sName Then Exit Function
        If ritem.Popup Then
            ' Recurse through any submenus
            If ritem.Child.Find(sName, ritem) Then Exit Function
        End If
    Next
    ' If we got all way through, it's not there
    Find = False
End Function

''@B Walk
Sub Walk(Optional vLevel As Variant)
    If IsMissing(vLevel) Then vLevel = 0
    Dim ritem As CMenuItem
    For Each ritem In nItems
        ' Walk through current list until user says stop
        If Not MenuWalker(ritem, CInt(vLevel)) Then Exit Sub
        ' Recurse through submenus
        If ritem.Popup Then ritem.Child.Walk vLevel + 1
    Next
End Sub
''@E Walk
    
''@B MenuWalker
#If fMenuWalker = 0 Then
Function MenuWalker(ritem As CMenuItem, iLevel As Integer) As Boolean
    Dim s As String
    s = "Name: " & ritem.Name & " ( "
    s = s & IIf(ritem.Disabled, "Disabled ", "")
    s = s & IIf(ritem.Checked, "Checked ", "")
    s = s & IIf(ritem.Grayed, "Grayed ", "")
    s = s & IIf(ritem.Popup, "Popup ", "") & ")"
    Debug.Print String$(iLevel, sTab) & s
    ' Your MenuWalker can return False to stop walk
    MenuWalker = True
End Function
#End If
''@E MenuWalker
'
