// BASEDISP.H
//
// Standard base dispatch interface from which to derive project-specific
// classes. This interface takes care of the boring grunt work required
// for all Visual Basic classes.
//

#ifndef _BaseDisp_
#define _BaseDisp_
#include "ERRINFO.H"
//#define DERIVE_BASE(X) CBaseIDispatch::Derive((CBaseIDispatch FAR *) (X), pDeriveFrom)
//#define PASS_BASE(X) (CBaseIDispatch FAR *) X
//#define GET_BASE CBaseIDispatch FAR * pDeriveFrom

CLASS CBaseIDispatch : public IDispatch
{
public:
	static void Derive(CBaseIDispatch FAR * pDest,
					   CBaseIDispatch FAR * pSource);

	// IUnknown methods
	STDMETHOD(QueryInterface)(REFIID riid, void FAR * FAR * ppvObj);
    STDMETHOD_(ULONG, AddRef)(void);
    STDMETHOD_(ULONG, Release)(void);

	// IDispatch methods
	STDMETHOD(GetTypeInfoCount)(UINT FAR * pctinfo);
	STDMETHOD(GetTypeInfo)(UINT itinfo, LCID lcid,
						   ITypeInfo FAR * FAR * pptinfo);
	STDMETHOD(GetIDsOfNames)(REFIID riid, OLECHAR FAR * FAR * rgszNames,
							 UINT cNames, LCID lcid, DISPID FAR * rgdispid);
	STDMETHOD(Invoke)(DISPID dispidMember, REFIID riid, LCID lcid,
					  WORD wFlags, DISPPARAMS FAR * pdispparams,
					  VARIANT FAR * pvarResult, EXCEPINFO FAR * pexcepinfo,
					  UINT FAR * puArgErr);

	// Constructor and Destructor
	CBaseIDispatch(void);
	virtual ~CBaseIDispatch(void);

protected:

	const IID FAR * m_pIID;
	CSupportErrorInfo FAR * m_pSupportErrorInfo;
	void RegisterObject(const IID FAR * lpclsid);

private:

	inline void QueryUnregisterObject(void);
	inline HRESULT LoadTypeInfo(void);

	ULONG m_dwRegister;
	ULONG m_RegisterCount;
	ITypeInfo FAR * m_ptinfo;
	ULONG m_refs;

};

CLASS CProvideClassInfo : public IProvideClassInfo
{
public:
	static HRESULT Create(ITypeInfo FAR * ptinfo,
						  CProvideClassInfo FAR * FAR * retVal);

	// IUnknown methods
	STDMETHOD(QueryInterface)(REFIID riid, void FAR * FAR * ppvObj);
    STDMETHOD_(ULONG, AddRef)(void);
	STDMETHOD_(ULONG, Release)(void);

	// IProvideClassInfo
	STDMETHOD(GetClassInfo)(THIS_ LPTYPEINFO FAR * ppTI);

	// Constructor and Destructor
	CProvideClassInfo();
	~CProvideClassInfo();

private:

	ITypeInfo FAR * m_ptinfo;
	unsigned long m_refs;

};

#endif // _BaseDisp_
