// SIEVE.CPP
//
// Implementation of the Sieve prime number class
//

#include "stdhdr.h"
#include "srvdata.h"
#include "sieve.h"

// Create method for class factory
CCSieve FAR* CCSieve::Create(void)
{
	CCSieve FAR* pobj = NULL;
	pobj = new CCSieve();
	if (pobj != NULL) {
		pobj->AddRef();
		pobj->RegisterObject(&CLSID_CCSieve);
	}
	return pobj;
}

// Constructor and destructor
CCSieve::CCSieve(void)
{
	m_pIID = &IID__ICSieve;

	// Set the m_SupportErrorInfo pointer for BaseDispatch.
	if (m_pSupportErrorInfo = new CSupportErrorInfo())
		m_pSupportErrorInfo->AddRef();
	m_ThrowInvalidMax = FALSE;
	m_ThrowInvalidArray = FALSE;

	//$B Construct
	// Default size is largest short
    m_af = 0;
    put_MaxPrime(SHRT_MAX - 1);
	m_sTitle = SysAllocString(OLESTR("Sieve Server"));
	//$E Construct

}

CCSieve::~CCSieve(void)
{
	//$B Destruct
	m_iMaxPrime = 0;	// Unnecessary, just a place for breakpoints
	//$E Destruct
}


// CSieve Properties

// NextPrime property
STDMETHODIMP CCSieve::get_NextPrime(THIS_ short FAR * piRet)
{
	//$B GetNextPrime

    // NextPrime = 0
    // Loop until we find a prime or overflow array
    m_iCur++;
    while (m_af[m_iCur]) {
	    m_iCur++;
		// Array overflow 
        if (m_iCur > m_iMaxPrime) {
			*piRet = 0;
			return NOERROR;
		}
	}

    // Cancel multiples of this prime
    int i;
    for (i = m_iCur + m_iCur; i < m_iMaxPrime; i += m_iCur) {
        m_af[i] = 1;
    }
    // Count and return it
    m_cPrime++;

	*piRet = m_iCur;
    return NOERROR;
	//$E GetNextPrime
}


// MaxPrime property
STDMETHODIMP CCSieve::get_MaxPrime(THIS_ short FAR * piRet)
{
	//$B GetMaxPrime
	*piRet = m_iMaxPrime;
    return NOERROR;
	//$E GetMaxPrime
}

STDMETHODIMP CCSieve::put_MaxPrime(THIS_ short iMaxPrime)
{
	//$B PutMaxPrime
	if (iMaxPrime <= 0) {
		m_pSupportErrorInfo->m_Desc = SysAllocString(OLESTR("Invalid maximum"));
		return ResultFromScode(MAKE_SCODE(SEVERITY_ERROR,
										  FACILITY_ITF, errInvalidMax));
	}
	m_iMaxPrime = iMaxPrime;
    return NOERROR;
	//$E PutMaxPrime
}

// Primes property
STDMETHODIMP CCSieve::get_Primes(THIS_ short FAR * piRet)
{
	//$B GetPrimes
	*piRet = m_cPrime;
    return NOERROR;
	//$E GetPrimes
}

// Title property
STDMETHODIMP CCSieve::get_Title(THIS_ BSTR FAR * psRet)
{
	//$B GetTitle
	*psRet = m_sTitle;
    return NOERROR;
	//$E GetTitle
}

STDMETHODIMP CCSieve::put_Title(THIS_ BSTR sTitle)
{
	//$B PutTitle
	if (SysReAllocString(&m_sTitle, sTitle) == FALSE) {
		m_pSupportErrorInfo->m_Desc = SysAllocString(OLESTR("Invalid string"));
		return ResultFromScode(MAKE_SCODE(SEVERITY_ERROR,
										  FACILITY_ITF, errInvalidString));
	}
    return NOERROR;
	//$E PutTitle
}

// Reinitialize method
STDMETHODIMP CCSieve::ReInitialize(THIS_ void)
{
	//$B Reinitialize
    if (m_af) {
		delete m_af;
	}
    m_af = new short[m_iMaxPrime];
	memset(m_af, 0, sizeof(short) * m_iMaxPrime);
    m_iCur = 1;
    m_cPrime = 0;
    return NOERROR;
	//$E Reinitialize
}

// AllPrimes method
STDMETHODIMP CCSieve::AllPrimes(THIS_ LPSAFEARRAY FAR * ppsa)
{
	//$B AllPrimes
	int i;
	long c;
	short FAR * af;
	SAFEARRAYBOUND asabound[1];
	
	// Validate array
	if (SafeArrayGetDim(*ppsa) != 1) 
		goto AllPrimesError;
	if (SafeArrayGetLBound(*ppsa, 1, &c) != S_OK)
		goto AllPrimesError;
	if (c != 0)
		goto AllPrimesError;
	if (SafeArrayGetUBound(*ppsa, 1, &c) != S_OK)
		goto AllPrimesError;
	if (c > SHRT_MAX - 1)
		goto AllPrimesError;
	if (SafeArrayGetElemsize(*ppsa) != sizeof(short)) 
		goto AllPrimesError;

	// Unlock direct access to data
	if (SafeArrayAccessData(*ppsa, (void HUGEP * FAR *)&af) != S_OK)
		goto AllPrimesError;

	// Maximum prime is the size of the array		
	m_iMaxPrime = (short)c;
    m_cPrime = 0;
    for (m_iCur = 2; m_iCur < m_iMaxPrime; m_iCur++) {
    	if (!af[m_iCur]) { 
        	// Found a prime
            for (i = m_iCur + m_iCur; i < m_iMaxPrime; i += m_iCur) {
                af[i] = 1;	// Cancel its multiples
            }
			// Write prime into position in array, count it
            af[m_cPrime] = m_iCur;
            m_cPrime++;
        }		
    }
	// Relock direct access to data
	if (SafeArrayUnaccessData(*ppsa) != S_OK)
		goto AllPrimesError;

	// Resize to include only the data
	asabound[0].lLbound = 0;
	asabound[0].cElements = m_cPrime;
	if (SafeArrayRedim(*ppsa, asabound) != S_OK)
		goto AllPrimesError;
    m_iCur = 1;
    return NOERROR;

AllPrimesError:

	m_pSupportErrorInfo->m_Desc = SysAllocString(OLESTR("Invalid array"));
	return ResultFromScode(MAKE_SCODE(SEVERITY_ERROR,
									  FACILITY_ITF, errInvalidArray));
	//$E AllPrimes
}
	
//
//	CCSieve Class Factory
//

// Constructor and destructor
CCSieveCF::CCSieveCF()
{ 
	m_refs = 0;
	g_cObj++;
}

CCSieveCF::~CCSieveCF()
{
	g_cObj--;
}

// Standard IUnknown methods
STDMETHODIMP
CCSieveCF::QueryInterface(REFIID riid, void FAR* FAR* ppv)
{
	if (IsEqualIID(riid, IID_IUnknown) ||
		IsEqualIID(riid, IID_IClassFactory)) {
		*ppv = this;
	} else {
		*ppv = NULL;
		return ResultFromScode(E_NOINTERFACE);
	}
	AddRef();
	return NOERROR;
}

STDMETHODIMP_(ULONG)
CCSieveCF::AddRef(void)
{
	return ++m_refs;
}


STDMETHODIMP_(ULONG)
CCSieveCF::Release(void)
{
	if (--m_refs == 0) {
		delete this;
		return 0;
	}
	return m_refs;
}

// Create a class object
STDMETHODIMP        
CCSieveCF::CreateInstance(IUnknown FAR* pUnkOuter, REFIID riid,
						  void FAR* FAR* ppv)
{
	IUnknown FAR *pcdt = NULL;
	HRESULT hr;

	//No aggregation
	if (pUnkOuter || !IIDOK(IUnknown)) {
		return ResultFromScode(CLASS_E_NOAGGREGATION);
	}

	pcdt = CCSieve::Create();
	if (pcdt == NULL) {
		*ppv = NULL;
		return ResultFromScode(E_FAIL);
	}
	hr = pcdt->QueryInterface(riid, ppv);
	pcdt->Release();
	return hr;
}

// Lock or unlock (server can only destruct when all objects are unlocked)
STDMETHODIMP
CCSieveCF::LockServer(BOOL fLock)
{
	fLock ? g_cLock++ : g_cLock--;
	return NOERROR;
}
