// SRVMAIN.CPP
//
// Main module of DLL server. This module references project-specific
// data, but does not // project-specific data even though its name is generic.
//

#include "stdhdr.h"
#include "srvdata.h"
#include "Sieve.h"

HINSTANCE g_hinst = 0;
ULONG g_cObj = 0;
ULONG g_cLock = 0;

#ifdef WIN32		
	
BOOL WINAPI
LibMain(HINSTANCE hinst, DWORD dwReason, LPVOID lpvReserved)
{
	switch (dwReason) {
	case DLL_PROCESS_ATTACH:
		// The DLL is being mapped into the process's address space
		g_hinst = hinst;

		// Do any additional 32-bit server initialization here

	   	break;

	case DLL_THREAD_ATTACH:
		// A thread is being created
	    break;

	case DLL_THREAD_DETACH:
		// A thread is exiting cleanly
	    break;

	case DLL_PROCESS_DETACH:
		// The DLL is being unmapped from the process's address space
		
		// Do any additional 32-bit server cleanup here
		g_hinst = 0;
				
	    break;
	}

	return TRUE;
}
#else

int PASCAL LibMain(HINSTANCE hinst, WORD wDataSeg, 
				   WORD cbHeapSize, LPSTR lpCmdLine)
{
	if (cbHeapSize != 0)
		UnlockData(0);
	g_hinst = hinst;

	// Do any additional 16-bit server initialization here
	
	return hinst;
}

extern "C"
int FAR PASCAL WEP(int bSystemExit)
{
	// Do any additional 16-bit server cleanup here

	return 1;
}            
#endif


const int CLOSEKEY = 1;
const int SAVEKEY = 0;

// DllCanUnloadNow.  Called by OLE to see if it can unload the DLL
STDAPI DllCanUnloadNow(void)
{
	if (g_cObj == 0L && g_cLock == 0L) {
		return ResultFromScode(S_OK);
	}
	return ResultFromScode(S_FALSE);
}

// By exporting DllRegisterServer, we can use regsvr[32].exe
STDAPI DllRegisterServer(void)
{
    ITypeLib FAR * ptlib = NULL;
    HRESULT hr;
	static char szDllName[MAX_PATH];
	LPOLESTR lpOleDllName;
	int cb;

    // If it wasn't registered, load typelib from the resource in DLL
	cb = GetModuleFileName(g_hinst, szDllName, MAX_PATH);
	lpOleDllName = OLESYSSTR(szDllName);
	if ((hr = LoadTypeLib(lpOleDllName, &ptlib)) != NOERROR) {
		FREESTR(lpOleDllName);
		return hr;
    } else {
		hr = RegisterTypeLib(ptlib, lpOleDllName, NULL);
		FREESTR(lpOleDllName);
        if (hr != NOERROR) {
            return hr;
		}
    }
    ptlib->Release();

	static char szKey[128];
	static char szVal[128];
	static char szTmp[10];
	CLSDATA FAR * pcls;
	HKEY hKey;

	for (pcls = acls; pcls->szCls[0] != '\0'; pcls++) {

		// Create Program ID
		// \Server.Class = ServerTitle
		strcat(strcat(strcpy(szKey, szLib), "."), pcls->szCls);
		hKey = RegisterItem(HKEY_CLASSES_ROOT, szKey, szTitle, SAVEKEY);

		// Write values to it
		// \Server.Class\CLSID = {CCCC}
		strcat(strcat(strcpy(szVal, "{"), pcls->szCLSID), "}");
		RegisterItem(hKey, "CLSID", szVal, CLOSEKEY);
		RegCloseKey(hKey);

		// Create class ID key
		// \CLSID\{CCCC} = ClassName (Ver x.y)
		strcat(strcat(strcpy(szKey, "CLSID\\{"), pcls->szCLSID), "}");
		strcat(strcpy(szVal, pcls->szCls), " (Ver ");
		strcat(strcat(szVal, itoa(iMajor, szTmp, 10)), ".");
		strcat(strcat(szVal, itoa(iMinor, szTmp, 10)), ")");
		hKey = RegisterItem(HKEY_CLASSES_ROOT, szKey, szVal, SAVEKEY);

		// Write values to it
		// \CLSID\{CCCC}\InprocServerZZ = d:\dir\server.dll
#ifdef WIN32		
		RegisterItem(hKey, "InprocServer32", szDllName, CLOSEKEY);
#else
		RegisterItem(hKey, "InprocServer", szDllName, CLOSEKEY);
#endif		
		// \CLSID\{CCCC}\ProgID = Server.Class
		strcat(strcat(strcpy(szVal, szLib), "."), pcls->szCls);
		RegisterItem(hKey, "ProgID", szVal, CLOSEKEY);
		RegCloseKey(hKey);

	}
	return S_OK;
}

// By exporting DllUnregisterServer, we can use regsvr[32] /u.
STDAPI DllUnregisterServer(void)
{
	static char szKey[128];
	CLSDATA FAR * pcls;
	LONG err;

	// Delete 
	// \TypeLib\{LLLL}
	strcat(strcat(strcpy(szKey, "TypeLib\\{"), szLIBID), "}");
	err = RegDeleteAllKey(HKEY_CLASSES_ROOT, szKey);

	for (pcls = acls; pcls->szCls[0] != '\0'; pcls++) {

		// Delete Program ID
		// \Server.Class = ServerTitle
		strcat(strcat(strcpy(szKey, szLib), "."), pcls->szCls);
		err = RegDeleteAllKey(HKEY_CLASSES_ROOT, szKey);

		// Delete class ID key
		// \CLSID\{CCCC} = ClassName (Ver x.y)
		strcat(strcat(strcpy(szKey, "CLSID\\{"), pcls->szCLSID), "}");
		err = RegDeleteAllKey(HKEY_CLASSES_ROOT, szKey);

		// Delete Interface key
		// \Interface\{IIII} = ClassName (Ver x.y)
		strcat(strcat(strcpy(szKey, "Interface\\{"), pcls->szIID), "}");
		err = RegDeleteAllKey(HKEY_CLASSES_ROOT, szKey);

	}
	return S_OK;
}

HRESULT LoadResourceTypeLib(REFIID libid, REFIID riid, ITypeInfo FAR* FAR* ppTInfo)
{
    ITypeLib FAR* ptlib = NULL;
    HRESULT hr;

	if (*ppTInfo)
		{
		(*ppTInfo)->Release();
		*ppTInfo = NULL;
		}
	//Version set at 1.0, check here if necessary.
	if( (hr = LoadRegTypeLib(libid, 1, 0, 0, &ptlib)) != NOERROR )
		{
		SYSCHAR szSysFileName[128];
		LPOLESTR lpOleFileName;
		// If it wasn't registered, load the typelib from the resource in the DLL.
		GetModuleFileName(g_hinst, szSysFileName, 128);
		lpOleFileName = OLESYSSTR(szSysFileName);
		if( (hr = LoadTypeLib(lpOleFileName, &ptlib)) != NOERROR)
			{
			FREESTR(lpOleFileName);
			return hr;
			}
		else
			{
			hr = RegisterTypeLib(ptlib, lpOleFileName, NULL);
			FREESTR(lpOleFileName);
			if (hr != NOERROR)
				return hr;
			}
		}

    hr = ptlib->GetTypeInfoOfGuid(riid, ppTInfo);
	ptlib->Release();
	return hr;
}


// Helper Functions

HKEY RegisterItem(HKEY hKey, const char * szKey, const char * szVal, 
				  int ordSave)
{
	HKEY hKeyOut;
#ifdef WIN32
	DWORD dwDisposition;
	RegCreateKeyEx(hKey, szKey, NULL, "", REG_OPTION_NON_VOLATILE,
				   KEY_ALL_ACCESS, NULL, &hKeyOut, &dwDisposition);
	RegSetValueEx(hKeyOut, NULL, NULL, REG_SZ,
				  (BYTE *)szVal, strlen(szVal) + 1);
#else
	RegCreateKey(hKey, szKey, &hKeyOut);
	RegSetValue(hKeyOut, NULL, REG_SZ, szVal, strlen(szVal) + 1);
#endif
	if (ordSave == CLOSEKEY)
		RegCloseKey(hKeyOut);
	return hKeyOut;
}

LONG RegDeleteAllKey(HKEY hKey, const char * szKey)
{
#if WIN32 
	char achKey[MAX_PATH];
	LONG err;
	HKEY hSubKey;
	
	// Try to delete whole thing--always works for Win95, but fails on 
	// keys with subkeys in WinNT
	err = RegDeleteKey(hKey, szKey);
	if (err == ERROR_SUCCESS) {
		return err;
	} else {

		err = RegOpenKeyEx(hKey, szKey, 0, KEY_ALL_ACCESS, &hSubKey);
		if (err != ERROR_SUCCESS) {
			return err;
		}

	    err = ERROR_SUCCESS; 
	    while (err == ERROR_SUCCESS) {
	        err = RegEnumKey(hSubKey, 0, achKey, MAX_PATH);
	        if (err == ERROR_SUCCESS) {
				err = RegDeleteAllKey(hSubKey, achKey);
			} 
	    }

		// Try to delete root again
		err = RegDeleteKey(hKey, szKey);
	}
	return err; 

#else
	// Win16 deletes all keys
	return RegDeleteKey(hKey, szKey);
#endif	
}
