VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CSystem"
Attribute VB_Creatable = True
Attribute VB_Exposed = False
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

'$ Uses REGISTRY.BAS

Private Type TSystemInfo
    iOemId As Long
    iPageSize As Long
    pMinimumApplicationAddress As Long
    pMaximumApplicationAddress As Long
    afActiveProcessorMask As Long
    iNumberOfProcessors As Long
    ordProcessorType As Long
    iAllocationGranularity As Long
    iReserved As Long
End Type

Private Type TMemoryStatus
    iLength As Long        ' sizeof(MEMORYSTATUS)
    iMemoryLoad As Long    ' percent of memory in use
    iTotalPhys As Long     ' bytes of physical memory
    iAvailPhys As Long     ' free physical memory bytes
    iTotalPageFile As Long ' bytes of paging file
    iAvailPageFile As Long ' free bytes of paging file
    iTotalVirtual As Long  ' user bytes of address space
    iAvailVirtual As Long  ' free user bytes
End Type

#If Win32 Then
Private Declare Sub GlobalMemoryStatus Lib "KERNEL32" ( _
    lpmem As TMemoryStatus)
Private Declare Sub GetSystemInfo Lib "KERNEL32" ( _
    sys As TSystemInfo)
#End If

Private iWinMajor As Integer
Private iWinMinor As Integer
Private iDosMajor As Integer
Private iDosMinor As Integer
Private fCoprocessor As Boolean
Private fPaging As Boolean
Private fUnder32 As Boolean
Private sProcessor As String
Private sMode As String
Private sWinDir As String
Private sSysDir As String
#If Win32 Then
Private mem As TMemoryStatus
Private sys As TSystemInfo
#End If

''@B Class_Initialize
Private Sub Class_Initialize()
    Dim dw As Long, c As Integer

#If Win32 Then
    dw = GetVersion()
    iWinMajor = dw And &HFF&
    iWinMinor = (dw And &HFF00&) / &H100&
    fUnder32 = True
    If dw And &H80000000 Then
        ' Windows 95 is 7.0
        iDosMinor = 0
        iDosMajor = 7
        sMode = "Windows 95"
    Else
        ' NT is 5.10
        iDosMinor = 10
        iDosMajor = 5
        sMode = "Windows NT"
    End If
    
    ' Emulator is integrated so that 32-bit always has coprocessor
    fCoprocessor = True
    ' 32-bit always has paging
    fPaging = True
    GetSystemInfo sys
    Select Case sys.ordProcessorType
    Case 386
        sProcessor = "386"
    Case 486
        sProcessor = "486"
    Case 586
        sProcessor = "586"
    Case 2000
        sProcessor = "R2000"
    Case 3000
        sProcessor = "R3000"
    Case 4000
        sProcessor = "R4000"
    Case 21064
        sProcessor = "A21064"
    Case Else
        sProcessor = "Unknown"
    End Select
#Else
    dw = GetVersion()
    iWinMajor = dw And &HFF&
    iWinMinor = (dw And &HFF00&) / &H100&
    iDosMinor = (dw And &HFF0000) / &H10000
    iDosMajor = (dw And &HF000000) / &H1000000
    dw = GetWinFlags()
    If dw And WF_STANDARD Then
        sMode = "Standard"
    Else
        sMode = "386 Enhanced"
    End If
    fCoprocessor = IIf(dw And WF_80x87, True, False)
    fPaging = IIf(dw And WF_PAGING, True, False)
    fUnder32 = False
    If dw And WF_WINNT Then
        fUnder32 = True
        sMode = "WOW " & sMode
    ElseIf iWinMajor = 3 And iWinMinor = 95 Then
        fUnder32 = True
        sMode = "WOW " & sMode
    End If
    If dw And WF_CPU286 Then
        sProcessor = "286"
    ElseIf dw And WF_CPU386 Then
        sProcessor = "386"
    ElseIf dw And WF_CPU486 Then
        sProcessor = "486"
    Else
        sProcessor = "Unknown"
    End If

#End If
    
    sWinDir = String$(cMaxPath, 0)
    c = GetWindowsDirectory(sWinDir, cMaxPath)
    sWinDir = Left(sWinDir, c)
    sSysDir = String$(cMaxPath, 0)
    c = GetSystemDirectory(sSysDir, cMaxPath)
    sSysDir = Left(sSysDir, c)

End Sub
''@E Class_Initialize

''@B GetFreeMemory
Property Get FreeMemory() As Long
#If Win32 Then
    mem.iLength = Len(mem)
    GlobalMemoryStatus mem
    FreeMemory = mem.iAvailPhys \ 1024
#Else
    FreeMemory = GetFreeSpace(0) \ 1024
#End If
End Property
''@E GetFreeMemory

''@B GetFreeResources
Property Get FreeResources() As Long
#If Win32 Then
    mem.iLength = Len(mem)
    GlobalMemoryStatus mem
    FreeResources = 100 - mem.iMemoryLoad
#Else
    FreeResources = GetFreeSystemResources(GFSR_SYSTEMRESOURCES)
#End If
End Property
''@E GetFreeResources

Property Get FreeUserResources() As Long
#If Win32 Then
    mem.iLength = Len(mem)
    GlobalMemoryStatus mem
    FreeUserResources = 100 - mem.iMemoryLoad
#Else
    FreeUserResources = GetFreeSystemResources(GFSR_USERRESOURCES)
#End If
End Property

Property Get FreeGdiResources() As Long
#If Win32 Then
    mem.iLength = Len(mem)
    GlobalMemoryStatus mem
    FreeGdiResources = 100 - mem.iMemoryLoad
#Else
    FreeGdiResources = GetFreeSystemResources(GFSR_GDIRESOURCES)
#End If
End Property

''@B GetWinMajor
Property Get WinMajor() As Integer
    WinMajor = iWinMajor
End Property
''@E GetWinMajor

Property Get WinMinor() As Integer
    WinMinor = iWinMinor
End Property

Property Get DosMajor() As Integer
    DosMajor = iDosMajor
End Property

Property Get DosMinor() As Integer
    DosMinor = iDosMinor
End Property

Property Get HasCoprocessor() As Boolean
    HasCoprocessor = fCoprocessor
End Property

Property Get HasPaging() As Boolean
    HasPaging = fPaging
End Property

Property Get IsUnder32() As Boolean
    IsUnder32 = fUnder32
End Property

Property Get Processor() As String
    Processor = sProcessor
End Property

Property Get Mode() As String
    Mode = sMode
End Property

Property Get WindowsDir() As String
    WindowsDir = sWinDir
End Property

Property Get SystemDir() As String
    SystemDir = sSysDir
End Property

Property Get USER() As String
#If Win32 Then
    Dim s As String, c As Long
    c = 80: s = String$(c + 1, 0)
    If GetUserName(s, c) = False Then
        USER = sEmpty
    Else
        USER = Left$(s, c)
    End If
#Else
    Dim s As String, c As Integer
    c = 80: s = String$(c + 1, 0)
    If WNetGetUser(s, c) <> 0 Then
        USER = sEmpty
    Else
        USER = Left$(s, InStr(s, sNullChr) - 1)
    End If
#End If
End Property

Property Get Machine() As String
    Dim s As String, c As Long
#If Win32 Then
    c = 16
    s = String$(16, 0)
    If GetComputerName(s, c) = False Then
        Machine = Left$(s, c)
    End If
#Else
    Machine = GetSetting("System", "Network", "ComputerName", "None")
#End If
End Property
